/*
 * LicenceTypesFrme.java
 *
 * Created on October 19, 2001, 7:10 PM
 */

package fina2.bank;

import java.awt.GridBagConstraints;
import java.util.Vector;

import javax.ejb.RemoveException;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.swing.AbstractAction;
import javax.swing.ListSelectionModel;

import org.apache.log4j.Logger;

import fina2.BaseFrame;
import fina2.Main;
import fina2.ui.UIManager;
import fina2.ui.table.EJBTable;
import fina2.ui.table.TableReviewFrame;

/**
 * 
 * @author Administrator
 */
public class LicenceTypesFrame extends BaseFrame {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;
	private fina2.Main main = fina2.Main.main;

	private EJBTable table;

	private LicenceTypesAmendAction amendAction;
	private LicenceTypesInsertAction insertAction;
	private LicenceTypesDeleteAction deleteAction;
	private LicenceTypesReviewAction reviewAction;

	private boolean canAmend = false;
	private boolean canDelete = false;
	private boolean canReview = false;

	/** Creates new form Banks */
	public LicenceTypesFrame() {
		ui.loadIcon("fina2.help", "help.gif");
		ui.loadIcon("fina2.print", "print.gif");
		ui.loadIcon("fina2.refresh", "refresh.gif");
		ui.loadIcon("fina2.close", "cancel.gif");

		ui.loadIcon("fina2.amend", "amend.gif");
		ui.loadIcon("fina2.create", "insert.gif");
		ui.loadIcon("fina2.delete", "delete.gif");

		table = new EJBTable();

		amendAction = new LicenceTypesAmendAction(main.getMainFrame(), table);
		insertAction = new LicenceTypesInsertAction(main.getMainFrame(), table);
		deleteAction = new LicenceTypesDeleteAction(main.getMainFrame(), table);
		reviewAction = new LicenceTypesReviewAction(main.getMainFrame(), table);

		table.addMouseListener(new java.awt.event.MouseListener() {
			public void mouseExited(java.awt.event.MouseEvent mouseEvent) {
			}

			public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {
			}

			public void mousePressed(java.awt.event.MouseEvent mouseEvent) {
			}

			public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {
				if (mouseEvent.getClickCount() > 1) {
					if (amendAction.isEnabled()) {
						amendAction.actionPerformed(null);
					}
				}
			}

			public void mouseEntered(java.awt.event.MouseEvent mouseEvent) {
			}
		});

		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.addSelectionListener(new javax.swing.event.ListSelectionListener() {
			public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
				if (table.getSelectedRow() == -1) {

					insertAction.setEnabled(canAmend);
					amendAction.setEnabled(false);
					reviewAction.setEnabled(false);
					deleteAction.setEnabled(false);
				} else {
					insertAction.setEnabled(canAmend);
					amendAction.setEnabled(canAmend);
					deleteAction.setEnabled(canDelete);
					reviewAction.setEnabled(canReview || canAmend);
				}
			}
		});

		initComponents();

		table.setPopupMenu(popupMenu);

		scrollPane.setViewportView(table);
		BaseFrame.ensureVisible(this);

	}

	private void initTable() {
		try {
			InitialContext jndi = fina2.Main.getJndiContext();
			Object ref = jndi.lookup("fina2/bank/BankSession");
			BankSessionHome home = (BankSessionHome) PortableRemoteObject.narrow(ref, BankSessionHome.class);

			BankSession session = home.create();

			Vector colNames = new Vector();
			colNames.add(ui.getString("fina2.description"));

			table.initTable(colNames, session.getLicenceTypesRows(main.getUserHandle(), main.getLanguageHandle()));

		} catch (Exception e) {
			Main.generalErrorHandler(e);
			ui.putConfigValue("fina2.bank.LicenceTypesFrame.visible", new Boolean(false));
		}
	}

	public void show() {
		if (isVisible())
			return;
		try {
			fina2.security.User user = (fina2.security.User) main.getUserHandle().getEJBObject();
			canAmend = user.hasPermission("fina2.bank.amend");
			canDelete = user.hasPermission("fina2.bank.delete");
			canReview = user.hasPermission("fina2.bank.review");
		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
		amendAction.setEnabled(canAmend);
		amendButton.setVisible(canAmend);
		amendItem.setVisible(canAmend);

		insertAction.setEnabled(canAmend);
		createButton.setVisible(canAmend);
		insertItem.setVisible(canAmend);

		deleteAction.setEnabled(canDelete);
		deleteButton.setVisible(canDelete);
		deleteItem.setVisible(canDelete);

		reviewAction.setEnabled(canAmend || canReview);
		reviewButton.setVisible(canAmend || canReview);
		printButton.setVisible(canAmend || canReview);
		reviewItem.setVisible(canAmend || canReview);

		initTable();

		super.show();

	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {// GEN-BEGIN:initComponents
		popupMenu = new javax.swing.JPopupMenu();
		amendItem = new javax.swing.JMenuItem();
		insertItem = new javax.swing.JMenuItem();
		deleteItem = new javax.swing.JMenuItem();
		reviewItem = new javax.swing.JMenuItem();
		jPanel1 = new javax.swing.JPanel();
		scrollPane = new javax.swing.JScrollPane();
		jPanel5 = new javax.swing.JPanel();
		jPanel6 = new javax.swing.JPanel();
		amendButton = new javax.swing.JButton();
		createButton = new javax.swing.JButton();
		deleteButton = new javax.swing.JButton();
		reviewButton = new javax.swing.JButton();
		jPanel2 = new javax.swing.JPanel();
		jPanel3 = new javax.swing.JPanel();
		jPanel4 = new javax.swing.JPanel();

		amendItem.setFont(ui.getFont());
		amendItem.setText("Item");
		amendItem.setAction(amendAction);
		popupMenu.add(amendItem);
		insertItem.setFont(ui.getFont());
		insertItem.setText("Item");
		insertItem.setAction(insertAction);
		popupMenu.add(insertItem);
		deleteItem.setFont(ui.getFont());
		deleteItem.setText("Item");
		deleteItem.setAction(deleteAction);
		popupMenu.add(deleteItem);
		reviewItem.setFont(ui.getFont());
		reviewItem.setText("Item");
		reviewItem.setAction(reviewAction);
		popupMenu.add(reviewItem);

		setTitle(ui.getString("fina2.bank.licenceTypesAction"));
		initBaseComponents();
		jPanel1.setLayout(new java.awt.BorderLayout());

		jPanel1.add(scrollPane, java.awt.BorderLayout.CENTER);

		jPanel5.setLayout(new java.awt.BorderLayout());

		jPanel5.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(1, 5, 1, 5)));
		jPanel6.setLayout(new java.awt.GridBagLayout());

		amendButton.setFont(ui.getFont());
		amendButton.setText("");
		amendButton.setAction(amendAction);
		jPanel6.add(amendButton, UIManager.getGridBagConstraints(0, 1, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		createButton.setFont(ui.getFont());
		createButton.setText("");
		createButton.setAction(insertAction);
		jPanel6.add(createButton, UIManager.getGridBagConstraints(0, 0, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		deleteButton.setFont(ui.getFont());
		deleteButton.setText("");
		deleteButton.setAction(deleteAction);
		jPanel6.add(deleteButton, UIManager.getGridBagConstraints(0, 2, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		reviewButton.setFont(ui.getFont());
		reviewButton.setText("");
		reviewButton.setAction(reviewAction);
		jPanel6.add(reviewButton, UIManager.getGridBagConstraints(0, 3, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		jPanel5.add(jPanel6, java.awt.BorderLayout.NORTH);

		jPanel1.add(jPanel5, java.awt.BorderLayout.EAST);

		getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

		jPanel2.setLayout(new java.awt.BorderLayout());

		if (ui.getHelpManager().IsHelpSystem()) {
			ui.getHelpManager().createDisplayHelpFromFocus(helpButton, "License_Types");
		} else {
			helpButton.setEnabled(false);
		}
		jPanel3.add(helpButton);

		jPanel2.add(jPanel3, java.awt.BorderLayout.WEST);

		jPanel4.add(printButton);

		jPanel4.add(refreshButton);

		jPanel4.add(closeButton);

		jPanel2.add(jPanel4, java.awt.BorderLayout.EAST);

		getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);

	}// GEN-END:initComponents

	protected void printButtonActionPerformed(java.awt.event.ActionEvent evt) {
		TableReviewFrame printFrame = new TableReviewFrame();
		printFrame.show(getTitle(), table);
	}

	protected void refreshButtonActionPerformed(java.awt.event.ActionEvent evt) {
		initTable();
	}

	protected void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {
		formComponentHidden(null);
		dispose();
	}

	protected void helpButtonActionPerformed(java.awt.event.ActionEvent evt) {

	}

	/** Exit the Application */
	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JPopupMenu popupMenu;
	private javax.swing.JMenuItem amendItem;
	private javax.swing.JMenuItem insertItem;
	private javax.swing.JMenuItem deleteItem;
	private javax.swing.JMenuItem reviewItem;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JScrollPane scrollPane;
	private javax.swing.JPanel jPanel5;
	private javax.swing.JPanel jPanel6;
	private javax.swing.JButton amendButton;
	private javax.swing.JButton createButton;
	private javax.swing.JButton deleteButton;
	private javax.swing.JButton reviewButton;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JPanel jPanel4;
	// End of variables declaration//GEN-END:variables

}

class LicenceTypesAmendAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;
	private LicenceTypeAmendDialog dialog;
	private java.awt.Frame parent;
	private EJBTable table;

	LicenceTypesAmendAction(java.awt.Frame parent, EJBTable table) {
		super();

		dialog = new LicenceTypeAmendDialog(parent, true);

		ui.loadIcon("fina2.amend", "amend.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, ui.getString("fina2.amend"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.amend"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		int index = table.getSelectedRow();

		dialog.show(table.getSelectedTableRow(), true);

		table.updateRow(index, dialog.getTableRow());
	}

}

class LicenceTypesInsertAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private LicenceTypeAmendDialog dialog;
	private java.awt.Frame parent;
	private EJBTable table;

	LicenceTypesInsertAction(java.awt.Frame parent, EJBTable table) {
		super();

		dialog = new LicenceTypeAmendDialog(parent, true);

		ui.loadIcon("fina2.insert", "insert.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, ui.getString("fina2.create"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.insert"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		dialog.show(null, true);
		if (dialog.getTableRow() != null)
			table.addRow(dialog.getTableRow());
	}

}

class LicenceTypesDeleteAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private java.awt.Frame parent;
	private EJBTable table;
	private LicenceType licenceType;

	LicenceTypesDeleteAction(java.awt.Frame parent, EJBTable table) {
		super();

		ui.loadIcon("fina2.delete", "delete.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, ui.getString("fina2.delete"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.delete"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		if (!ui.showConfirmBox(parent, ui.getString("fina2.bank.licenceTypeDeleteQuestion")))
			return;

		try {
			int index = table.getSelectedRow();

			InitialContext jndi = fina2.Main.getJndiContext();
			Object ref = jndi.lookup("fina2/bank/LicenceType");
			LicenceTypeHome home = (LicenceTypeHome) PortableRemoteObject.narrow(ref, LicenceTypeHome.class);

			licenceType = home.findByPrimaryKey((LicenceTypePK) table.getSelectedPK());
			licenceType.remove();
			table.removeRow(index);
		} catch (RemoveException ex) {
			Logger log = Logger.getLogger(getClass());
			log.error(ex.getMessage(), ex);
			ui.showMessageBox(null, ui.getString("fina2.license.licenseIsUsed") + " " + ex.getMessage());
		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}

	}
}

class LicenceTypesReviewAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private LicenceTypeAmendDialog dialog;
	private java.awt.Frame parent;
	private EJBTable table;

	LicenceTypesReviewAction(java.awt.Frame parent, EJBTable table) {
		super();

		dialog = new LicenceTypeAmendDialog(parent, true);
		ui.loadIcon("fina2.review", "review.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, ui.getString("fina2.review"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.review"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		dialog.show(table.getSelectedTableRow(), false);
	}

}
