/*
 * ManagingBodiesFrame.java
 *
 * Created on 1  2002 ., 10:51
 */

package fina2.bank;

import java.awt.GridBagConstraints;
import java.util.Vector;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.swing.AbstractAction;
import javax.swing.ListSelectionModel;

import fina2.BaseFrame;
import fina2.Main;
import fina2.ui.UIManager;
import fina2.ui.table.EJBTable;
import fina2.ui.table.TableReviewFrame;

/**
 * 
 * @author Vasop
 */
public class ManagingBodiesFrame extends BaseFrame {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;
	private fina2.Main main = fina2.Main.main;

	private EJBTable table;

	private ManagingBodyAmendAction amendAction;
	private ManagingBodyInsertAction insertAction;
	private ManagingBodyReviewAction reviewAction;
	private ManagingBodyDeleteAction deleteAction;

	private boolean canAmend = false;
	private boolean canDelete = false;
	private boolean canReview = false;

	/** Creates new form ManagingBodiesFrame */
	public ManagingBodiesFrame() {

		ui.loadIcon("fina2.help", "help.gif");
		ui.loadIcon("fina2.print", "print.gif");
		ui.loadIcon("fina2.refresh", "refresh.gif");
		ui.loadIcon("fina2.close", "cancel.gif");

		ui.loadIcon("fina2.amend", "amend.gif");
		ui.loadIcon("fina2.create", "insert.gif");
		ui.loadIcon("fina2.review", "review.gif");
		ui.loadIcon("fina2.delete", "delete.gif");

		table = new EJBTable();

		amendAction = new ManagingBodyAmendAction(main.getMainFrame(), table);
		insertAction = new ManagingBodyInsertAction(main.getMainFrame(), table);
		reviewAction = new ManagingBodyReviewAction(main.getMainFrame(), table);
		deleteAction = new ManagingBodyDeleteAction(main.getMainFrame(), table);

		table.addMouseListener(new java.awt.event.MouseListener() {
			public void mouseExited(java.awt.event.MouseEvent mouseEvent) {
			}

			public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {
			}

			public void mousePressed(java.awt.event.MouseEvent mouseEvent) {
			}

			public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {
				if (mouseEvent.getClickCount() == 2) {
					if (amendAction.isEnabled()) {
						amendAction.actionPerformed(null);
					} else {
						if (reviewAction.isEnabled()) {
							reviewAction.actionPerformed(null);
						}
					}
				}
			}

			public void mouseEntered(java.awt.event.MouseEvent mouseEvent) {
			}
		});

		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.addSelectionListener(new javax.swing.event.ListSelectionListener() {
			public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
				if (table.getSelectedRow() == -1) {
					insertAction.setEnabled(canAmend);
					amendAction.setEnabled(false);
					reviewAction.setEnabled(false);
					deleteAction.setEnabled(false);
				} else {
					insertAction.setEnabled(canAmend);
					amendAction.setEnabled(canAmend);
					deleteAction.setEnabled(canDelete);
					reviewAction.setEnabled(canReview || canAmend);
				}
			}
		});

		initComponents();
		// table.setPopupMenu(popupMenu);
		scrollPane.setViewportView(table);
		BaseFrame.ensureVisible(this);
	}

	private void initTable() {
		try {

			InitialContext jndi = fina2.Main.getJndiContext();
			Object ref = jndi.lookup("fina2/bank/ManagingBodySession");
			ManagingBodySessionHome home = (ManagingBodySessionHome) PortableRemoteObject.narrow(ref, ManagingBodySessionHome.class);

			ManagingBodySession session = home.create();

			Vector colNames = new Vector();
			colNames.add(ui.getString("fina2.bank.managingBody"));

			table.initTable(colNames, session.getManagingBodyRows(main.getUserHandle(), main.getLanguageHandle()));

		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
	}

	public void show() {
		if (isVisible())
			return;

		try {
			fina2.security.User user = (fina2.security.User) main.getUserHandle().getEJBObject();
			canAmend = user.hasPermission("fina2.bank.amend");
			canDelete = user.hasPermission("fina2.bank.delete");
			canReview = user.hasPermission("fina2.bank.review");
		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
		amendAction.setEnabled(canAmend);
		amendButton.setVisible(canAmend);
		amendItem.setVisible(canAmend);

		insertAction.setEnabled(canAmend);
		createButton.setVisible(canAmend);
		insertItem.setVisible(canAmend);

		deleteAction.setEnabled(canDelete);
		deleteButton.setVisible(canDelete);
		deleteItem.setVisible(canDelete);

		reviewAction.setEnabled(canAmend || canReview);
		reviewButton.setVisible(canAmend || canReview);
		printButton.setVisible(canAmend || canReview);
		reviewItem.setVisible(canAmend || canReview);

		initTable();

		super.show();
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {// GEN-BEGIN:initComponents
		popupMenu = new javax.swing.JPopupMenu();
		amendItem = new javax.swing.JMenuItem();
		insertItem = new javax.swing.JMenuItem();
		reviewItem = new javax.swing.JMenuItem();
		deleteItem = new javax.swing.JMenuItem();
		jPanel1 = new javax.swing.JPanel();
		scrollPane = new javax.swing.JScrollPane();
		jPanel3 = new javax.swing.JPanel();
		jPanel6 = new javax.swing.JPanel();
		createButton = new javax.swing.JButton();
		amendButton = new javax.swing.JButton();
		reviewButton = new javax.swing.JButton();
		deleteButton = new javax.swing.JButton();
		jPanel2 = new javax.swing.JPanel();
		jPanel4 = new javax.swing.JPanel();
		jPanel5 = new javax.swing.JPanel();

		amendItem.setFont(ui.getFont());
		amendItem.setText("Item");
		amendItem.setAction(amendAction);
		popupMenu.add(amendItem);

		insertItem.setFont(ui.getFont());
		insertItem.setText("Item");
		insertItem.setAction(insertAction);
		popupMenu.add(insertItem);

		reviewItem.setFont(ui.getFont());
		reviewItem.setAction(reviewAction);
		popupMenu.add(reviewItem);

		deleteItem.setFont(ui.getFont());
		deleteItem.setText("Item");
		deleteItem.setAction(deleteAction);
		popupMenu.add(deleteItem);
		initBaseComponents();
		jPanel1.setLayout(new java.awt.BorderLayout());

		jPanel1.add(scrollPane, java.awt.BorderLayout.CENTER);

		jPanel3.setLayout(new java.awt.BorderLayout());

		jPanel6.setLayout(new java.awt.GridBagLayout());

		createButton.setIcon(ui.getIcon("fina2.create"));
		createButton.setFont(ui.getFont());
		createButton.setText(ui.getString("fina2.create"));
		createButton.setAction(insertAction);
		jPanel6.add(createButton, UIManager.getGridBagConstraints(-1, -1, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		amendButton.setIcon(ui.getIcon("fina2.amend"));
		amendButton.setFont(ui.getFont());
		amendButton.setText(ui.getString("fina2.amend"));
		amendButton.setAction(amendAction);
		jPanel6.add(amendButton, UIManager.getGridBagConstraints(0, 1, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		reviewButton.setIcon(ui.getIcon("fina2.review"));
		reviewButton.setFont(ui.getFont());
		reviewButton.setText(ui.getString("fina2.review"));
		reviewButton.setAction(reviewAction);
		jPanel6.add(reviewButton, UIManager.getGridBagConstraints(0, 2, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		deleteButton.setIcon(ui.getIcon("fina2.delete"));
		deleteButton.setFont(ui.getFont());
		deleteButton.setText(ui.getString("fina2.delete"));
		deleteButton.setAction(deleteAction);
		jPanel6.add(deleteButton, UIManager.getGridBagConstraints(0, 3, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		jPanel3.add(jPanel6, java.awt.BorderLayout.NORTH);

		jPanel1.add(jPanel3, java.awt.BorderLayout.EAST);

		getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

		jPanel2.setLayout(new java.awt.BorderLayout());

		if (ui.getHelpManager().IsHelpSystem()) {
			ui.getHelpManager().createDisplayHelpFromFocus(helpButton, "Management");
		} else {
			helpButton.setEnabled(false);
		}

		jPanel4.add(helpButton);

		jPanel2.add(jPanel4, java.awt.BorderLayout.WEST);

		jPanel5.add(printButton);

		jPanel5.add(refreshButton);

		jPanel5.add(closeButton);

		jPanel2.add(jPanel5, java.awt.BorderLayout.EAST);

		getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);

	}// GEN-END:initComponents

	protected void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {
		formComponentHidden(null);
		dispose();
	}

	protected void refreshButtonActionPerformed(java.awt.event.ActionEvent evt) {
		initTable();
	}

	protected void printButtonActionPerformed(java.awt.event.ActionEvent evt) {
		TableReviewFrame printFrame = new TableReviewFrame();
		printFrame.show(getTitle(), table);
	}

	protected void helpButtonActionPerformed(java.awt.event.ActionEvent evt) {

	}

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton amendButton;
	private javax.swing.JMenuItem amendItem;
	private javax.swing.JButton createButton;
	private javax.swing.JButton deleteButton;
	private javax.swing.JMenuItem deleteItem;
	private javax.swing.JMenuItem insertItem;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JPanel jPanel4;
	private javax.swing.JPanel jPanel5;
	private javax.swing.JPanel jPanel6;
	private javax.swing.JPopupMenu popupMenu;
	private javax.swing.JButton reviewButton;
	private javax.swing.JMenuItem reviewItem;
	private javax.swing.JScrollPane scrollPane;
	// End of variables declaration//GEN-END:variables
}

class ManagingBodyAmendAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private ManagingBodyAmendDialog dialog;
	private java.awt.Frame parent;
	private EJBTable table;

	ManagingBodyAmendAction(java.awt.Frame parent, EJBTable table) {
		super();

		dialog = new ManagingBodyAmendDialog(parent, true);

		ui.loadIcon("fina2.amend", "amend.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, ui.getString("fina2.amend"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.amend"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		int index = table.getSelectedRow();

		dialog.show(table.getSelectedTableRow(), true);

		table.updateRow(index, dialog.getTableRow());
	}

}

class ManagingBodyInsertAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private ManagingBodyAmendDialog dialog;
	private java.awt.Frame parent;
	private EJBTable table;

	ManagingBodyInsertAction(java.awt.Frame parent, EJBTable table) {
		super();

		dialog = new ManagingBodyAmendDialog(parent, true);

		ui.loadIcon("fina2.insert", "insert.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, ui.getString("fina2.create"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.insert"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		dialog.show(null, true);
		if (dialog.getTableRow() != null)
			table.addRow(dialog.getTableRow());
	}

}

class ManagingBodyReviewAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private ManagingBodyAmendDialog dialog;
	private java.awt.Frame parent;
	private EJBTable table;

	ManagingBodyReviewAction(java.awt.Frame parent, EJBTable table) {
		super();

		dialog = new ManagingBodyAmendDialog(parent, true);
		ui.loadIcon("fina2.review", "review.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, ui.getString("fina2.review"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.review"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		dialog.show(table.getSelectedTableRow(), false);
	}

}

class ManagingBodyDeleteAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private java.awt.Frame parent;
	private EJBTable table;
	private ManagingBody managingBody;

	ManagingBodyDeleteAction(java.awt.Frame parent, EJBTable table) {
		super();

		ui.loadIcon("fina2.delete", "delete.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, ui.getString("fina2.delete"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.delete"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		if (!ui.showConfirmBox(parent, ui.getString("fina2.bank.managingBodyDeleteQuestion")))
			return;

		try {
			int index = table.getSelectedRow();

			InitialContext jndi = fina2.Main.getJndiContext();
			Object ref = jndi.lookup("fina2/bank/ManagingBody");
			ManagingBodyHome home = (ManagingBodyHome) PortableRemoteObject.narrow(ref, ManagingBodyHome.class);

			managingBody = home.findByPrimaryKey((ManagingBodyPK) table.getSelectedPK());
			managingBody.remove();

			table.removeRow(index);
		} catch (Exception e) {
			ui.showMessageBox(parent, ui.getString("fina2.delete.managingBody"));
			e.printStackTrace();
			Main.errorHandler(parent, Main.getString("fina2.title"), Main.getString("fina2.delete.managingBody"));
		}

	}

}
