/*
 * SelectBankGroupDialog.java
 *
 * Created on December 13, 2001, 9:00 PM
 */

package fina2.bank;

import java.util.Vector;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.swing.ListSelectionModel;

import fina2.Main;
import fina2.ui.table.EJBTable;
import fina2.ui.table.TableRow;

/**
 *
 * @author  Kakha Kakhidze
 */
public class SelectBankGroupDialog extends javax.swing.JDialog {

    private fina2.ui.UIManager ui = fina2.Main.main.ui;
    private fina2.Main main = fina2.Main.main;

    private EJBTable table;
    private TableRow tableRow;

    /** Creates new form SelectBankGroupDialog */
    public SelectBankGroupDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);

        ui.loadIcon("fina2.cancel", "cancel.gif");
        ui.loadIcon("fina2.help", "help.gif");
        ui.loadIcon("fina2.ok", "ok.gif");

        table = new EJBTable();

        initComponents();

        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table
                .addSelectionListener(new javax.swing.event.ListSelectionListener() {
                    public void valueChanged(
                            javax.swing.event.ListSelectionEvent evt) {
                        if (table.getSelectedRow() == -1) {
                            okButton.setEnabled(false);
                        } else {
                            okButton.setEnabled(true);
                        }
                    }
                });

        table.addMouseListener(new java.awt.event.MouseListener() {
            public void mouseExited(java.awt.event.MouseEvent mouseEvent) {
            }

            public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {
            }

            public void mousePressed(java.awt.event.MouseEvent mouseEvent) {
            }

            public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {
                if (mouseEvent.getClickCount() == 2) {
                    if (table.getSelectedRow() != -1) {
                        okButtonActionPerformed(null);
                    }
                }
            }

            public void mouseEntered(java.awt.event.MouseEvent mouseEvent) {
            }
        });

        scrollPane.setViewportView(table);
    }

    public TableRow getTableRow() {
        return tableRow;
    }

    public void show() {
        initTable();

        setLocationRelativeTo(getParent());
        super.show();
    }

    private void initTable() {
        try {
            InitialContext jndi = fina2.Main.getJndiContext();
            Object ref = jndi.lookup("fina2/bank/BankSession");
            BankSessionHome home = (BankSessionHome) PortableRemoteObject
                    .narrow(ref, BankSessionHome.class);

            BankSession session = home.create();

            Vector colNames = new Vector();
            colNames.add(ui.getString("fina2.code"));
            colNames.add(ui.getString("fina2.description"));

            table.initTable(colNames, session.getBankGroupsRows(main
                    .getUserHandle(), main.getLanguageHandle()));

        } catch (Exception e) {
            Main.generalErrorHandler(e);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jPanel1 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        helpButton = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        scrollPane = new javax.swing.JScrollPane();

        setTitle(ui.getString("fina2.bank.bankGroups"));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jPanel1.setLayout(new java.awt.BorderLayout());

        helpButton.setIcon(ui.getIcon("fina2.help"));
        helpButton.setFont(ui.getFont());
        helpButton.setText(ui.getString("fina2.help"));
        helpButton.setEnabled(false);
        jPanel2.add(helpButton);

        jPanel1.add(jPanel2, java.awt.BorderLayout.WEST);

        okButton.setIcon(ui.getIcon("fina2.ok"));
        okButton.setFont(ui.getFont());
        okButton.setText(ui.getString("fina2.ok"));
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        jPanel3.add(okButton);

        cancelButton.setIcon(ui.getIcon("fina2.cancel"));
        cancelButton.setFont(ui.getFont());
        cancelButton.setText(ui.getString("fina2.cancel"));
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        jPanel3.add(cancelButton);

        jPanel1.add(jPanel3, java.awt.BorderLayout.EAST);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        scrollPane.setPreferredSize(new java.awt.Dimension(320, 200));
        getContentPane().add(scrollPane, java.awt.BorderLayout.CENTER);

        pack();
    }//GEN-END:initComponents

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        tableRow = null;
        dispose();
    }//GEN-LAST:event_cancelButtonActionPerformed

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
        tableRow = table.getSelectedTableRow();
        dispose();
    }//GEN-LAST:event_okButtonActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        tableRow = null;
        dispose();
    }//GEN-LAST:event_closeDialog

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JButton helpButton;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JButton okButton;
    private javax.swing.JButton cancelButton;
    private javax.swing.JScrollPane scrollPane;
    // End of variables declaration//GEN-END:variables

}
