package fina2.i18n;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.text.html.HTML;

import fina2.BaseFrame;
import fina2.FinaTypeException;
import fina2.FinaTypeException.Type;
import fina2.Main;
import fina2.ui.UIManager;
import fina2.ui.table.EJBTable;
import fina2.ui.table.TableRow;

public class LanguagesFrame extends BaseFrame {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;
	private fina2.Main main = fina2.Main.main;
	private static int oldDefaultLangiage;
	private EJBTable table;

	private boolean canAmend = false;

	private LanguageCreateAction createAction;
	private LanguageAmendAction amendAction;
	private LanguageReviewAction reviewAction;
	private LanguageDeleteAction deleteAction;
	private BundlesAmendAction bandlesAmendAction;
	private DefaultButtonAction defaultButtonAction;

	/** Creates new form LanguagesFrame */

	public LanguagesFrame() {
		ui.loadIcon("fina2.help", "help.gif");
		ui.loadIcon("fina2.close", "cancel.gif");

		table = new EJBTable();

		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		createAction = new LanguageCreateAction(main.getMainFrame(), table);
		amendAction = new LanguageAmendAction(main.getMainFrame(), table);
		reviewAction = new LanguageReviewAction(main.getMainFrame(), table);
		deleteAction = new LanguageDeleteAction(main.getMainFrame(), table);
		bandlesAmendAction = new BundlesAmendAction(main.getMainFrame(), table);
		defaultButtonAction = new DefaultButtonAction(main.getMainFrame(), table);
		initComponents();

		table.setPopupMenu(tableMenu);
		scrollPane.setViewportView(table);
		BaseFrame.ensureVisible(this);
	}

	public void initTable() {
		try {
			fina2.security.User user = (fina2.security.User) main.getUserHandle().getEJBObject();
			canAmend = user.hasPermission("fina2.security.amend");
			if (!canAmend) {
				ui.putConfigValue("fina2.i18n.LanguagesFrame.visible", new Boolean(false));
				throw new FinaTypeException(Type.PERMISSIONS_DENIED, new String[] { "fina2.security.amend" });
			}

			InitialContext jndi = fina2.Main.getJndiContext();
			Object ref = jndi.lookup("fina2/i18n/LanguageSession");
			LanguageSessionHome home = (LanguageSessionHome) PortableRemoteObject.narrow(ref, LanguageSessionHome.class);

			LanguageSession session = home.create();

			Vector colNames = new Vector();
			colNames.add(ui.getString("fina2.code"));
			colNames.add(ui.getString("fina2.description"));
			table.initTable(colNames, session.getLanguagesRows(main.getUserHandle(), main.getLanguageHandle()));
			table.getValueAt(table.getSelectedRow(), 0);
			if (ui.getDefaultLanguage() != null) {
				table.setDefaultRenderer(Object.class, new LanguageTableCellRenderer());
				table.fireSelectionChangedEvent();
				table.selectAll();
				table.fireSelectionChangedEvent();
				table.setSelectionMode(1);
			}
		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
	}

	public void show() {
		if (isVisible()) {
			requestFocus();
			return;
		}
		initTable();
		super.show();
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {// GEN-BEGIN:initComponents
		tableMenu = new javax.swing.JPopupMenu();
		jMenuItem1 = new javax.swing.JMenuItem();
		jPanel1 = new javax.swing.JPanel();
		jPanel4 = new javax.swing.JPanel();
		helpButton = new javax.swing.JButton();
		jPanel5 = new javax.swing.JPanel();
		refreshButton = new javax.swing.JButton();
		closeButton = new javax.swing.JButton();
		jPanel2 = new javax.swing.JPanel();
		jPanel3 = new javax.swing.JPanel();
		jPanel6 = new javax.swing.JPanel();
		createButton = new javax.swing.JButton();
		amendButton = new javax.swing.JButton();
		reviewButton = new javax.swing.JButton();
		deleteButton = new javax.swing.JButton();
		bandlesAmendButton = new javax.swing.JButton();
		scrollPane = new javax.swing.JScrollPane();
		defaultButton = new JButton("hello wolrd");
		jMenuItem1.setFont(ui.getFont());
		jMenuItem1.setText("Item");
		jMenuItem1.setAction(createAction);
		tableMenu.add(jMenuItem1);

		this.setFont(ui.getFont());

		setTitle(ui.getString("fina2.i18n.languages"));

		jPanel1.setLayout(new java.awt.BorderLayout());
		initBaseComponents();

		jPanel4.add(helpButton);
		if (ui.getHelpManager().IsHelpSystem()) {
			ui.getHelpManager().createDisplayHelpFromFocus(helpButton, "Languages");
		} else {
			helpButton.setEnabled(false);
		}

		jPanel1.add(jPanel4, java.awt.BorderLayout.WEST);

		jPanel5.add(refreshButton);

		jPanel5.add(closeButton);

		jPanel1.add(jPanel5, java.awt.BorderLayout.EAST);

		getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

		jPanel2.setLayout(new java.awt.BorderLayout());

		jPanel3.setLayout(new java.awt.BorderLayout());

		jPanel3.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(1, 5, 1, 5)));
		jPanel6.setLayout(new java.awt.GridBagLayout());

		createButton.setFont(ui.getFont());
		createButton.setAction(createAction);
		jPanel6.add(createButton, UIManager.getGridBagConstraints(0, 0, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		amendButton.setFont(ui.getFont());
		amendButton.setAction(amendAction);
		jPanel6.add(amendButton, UIManager.getGridBagConstraints(0, 1, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		reviewButton.setFont(ui.getFont());
		reviewButton.setAction(reviewAction);
		jPanel6.add(reviewButton, UIManager.getGridBagConstraints(0, 2, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		deleteButton.setFont(ui.getFont());
		deleteButton.setAction(deleteAction);
		jPanel6.add(deleteButton, UIManager.getGridBagConstraints(0, 3, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		bandlesAmendButton.setFont(ui.getFont());
		bandlesAmendButton.setAction(bandlesAmendAction);
		jPanel6.add(bandlesAmendButton, UIManager.getGridBagConstraints(0, 4, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));

		defaultButton.setFont(ui.getFont());
		defaultButton.setAction(defaultButtonAction);
		if (!(ui.getDefaultLanguage() != null))
			defaultButton.setEnabled(false);
		jPanel6.add(defaultButton, UIManager.getGridBagConstraints(0, 5, -1, -1, -1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(5, 0, 0, 0)));
		jPanel3.add(jPanel6, java.awt.BorderLayout.NORTH);

		jPanel2.add(jPanel3, java.awt.BorderLayout.EAST);

		jPanel2.add(scrollPane, java.awt.BorderLayout.CENTER);

		getContentPane().add(jPanel2, java.awt.BorderLayout.CENTER);

	}// GEN-END:initComponents

	//

	public void refreshButtonActionPerformed(ActionEvent evt) {
		initTable();
	}

	protected void closeButtonActionPerformed(ActionEvent evt) {
		formComponentHidden(null);
		dispose();
	}

	protected void helpButtonActionPerformed(java.awt.event.ActionEvent evt) {

	}

	/** Exit the Application */

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton amendButton;
	private javax.swing.JButton createButton;
	private javax.swing.JButton deleteButton;
	private javax.swing.JButton bandlesAmendButton;
	private javax.swing.JMenuItem jMenuItem1;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JPanel jPanel4;
	private javax.swing.JPanel jPanel5;
	private javax.swing.JPanel jPanel6;
	private javax.swing.JButton reviewButton;
	private javax.swing.JScrollPane scrollPane;
	private javax.swing.JPopupMenu tableMenu;

	// default button

	javax.swing.JButton defaultButton;
	// End of variables declaration//GEN-END:variables
}

class LanguageCreateAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private LanguageAmendDialog dialog;
	private java.awt.Frame parent;
	private EJBTable table;

	public LanguageCreateAction() {
	}

	LanguageCreateAction(java.awt.Frame parent, EJBTable table) {
		super();

		dialog = new LanguageAmendDialog(parent, true);

		ui.loadIcon("fina2.create", "insert.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, ui.getString("fina2.create"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.create"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		dialog.show(null, true);
		if (dialog.getTableRow() != null)
			table.addRow(dialog.getTableRow());
	}

}

class LanguageTableCellRenderer extends DefaultTableCellRenderer {
	UIManager ui = new UIManager();

	public Component getTableCellRendererComponent(JTable table, Object obj, boolean isSelected, boolean hasFocus, int row, int column) {
		Component cell = super.getTableCellRendererComponent(table, obj, isSelected, hasFocus, row, column);

		String defaultlanguage = ui.getDefaultLanguage();
		if (isSelected) {
			cell.setForeground(java.awt.Color.white);
		}
		if (table.getValueAt(row, 0).equals(defaultlanguage)) {
			cell.setForeground(java.awt.Color.green);
		} else if (!isSelected) {
			cell.setForeground(java.awt.Color.black);
		}
		return cell;
	}
}

class DefaultButtonAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private java.awt.Frame parent;
	private EJBTable table;

	DefaultButtonAction(java.awt.Frame parent, EJBTable table) {
		super();
		this.parent = parent;
		this.table = table;
		ui.loadIcon("fina2.default", "default.gif");
		putValue(AbstractAction.NAME, ui.getString("fina2.default"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.default"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		int selectedrow = table.getSelectedRow();

		ui.setDefaultLanguage((String) table.getValueAt(selectedrow, 0));
		table.fireSelectionChangedEvent();
		table.selectAll();

		table.fireSelectionChangedEvent();
		table.setSelectionMode(1);

	}
}

class LanguageAmendAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private LanguageAmendDialog dialog;
	private java.awt.Frame parent;
	private EJBTable table;

	LanguageAmendAction(java.awt.Frame parent, EJBTable table) {
		super();

		dialog = new LanguageAmendDialog(parent, true);

		ui.loadIcon("fina2.amend", "amend.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, ui.getString("fina2.amend"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.amend"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		int index = table.getSelectedRow();

		dialog.show(table.getSelectedTableRow(), true);

		table.updateRow(index, dialog.getTableRow());
	}

}

class LanguageReviewAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private LanguageAmendDialog dialog;
	private java.awt.Frame parent;
	private EJBTable table;

	LanguageReviewAction(java.awt.Frame parent, EJBTable table) {
		super();

		dialog = new LanguageAmendDialog(parent, true);

		ui.loadIcon("fina2.review", "review.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, ui.getString("fina2.review"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.review"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		dialog.show(table.getSelectedTableRow(), false);
	}

}

class LanguageDeleteAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private java.awt.Frame parent;
	private EJBTable table;

	LanguageDeleteAction(java.awt.Frame parent, EJBTable table) {
		super();

		ui.loadIcon("fina2.delete", "delete.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, ui.getString("fina2.delete"));
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.delete"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		if (!ui.showConfirmBox(parent, ui.getString("fina2.i18n.languageDeleteQuestion")))
			return;

		try {
			int index = table.getSelectedRow();

			InitialContext jndi = fina2.Main.getJndiContext();
			Object ref = jndi.lookup("fina2/i18n/Language");
			LanguageHome home = (LanguageHome) PortableRemoteObject.narrow(ref, LanguageHome.class);

			Language lang = home.findByPrimaryKey((LanguagePK) table.getSelectedPK());
			lang.remove();

			table.removeRow(index);
		} catch (Exception e) {
			Main.errorHandler(parent, Main.getString("fina2.title"), Main.getString("fina2.delete.language"));
		}
	}

}

class BundlesAmendAction extends AbstractAction {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;

	private java.awt.Frame parent;
	private EJBTable table;

	public BundlesAmendAction(java.awt.Frame parent, EJBTable table) {
		super();

		ui.loadIcon("fina2.bundles", "bundles.gif");

		this.parent = parent;
		this.table = table;

		putValue(AbstractAction.NAME, "Bundles");
		putValue(AbstractAction.SMALL_ICON, ui.getIcon("fina2.bundles"));
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		try {
			Collection<TableRow> tableRows = table.getRows();
			List<Language> languages = new ArrayList<Language>();

			for (TableRow row : tableRows) {
				InitialContext jndi = fina2.Main.getJndiContext();
				Object ref = jndi.lookup("fina2/i18n/Language");
				LanguageHome home = (LanguageHome) PortableRemoteObject.narrow(ref, LanguageHome.class);
				languages.add(home.findByPrimaryKey((LanguagePK) row.getPrimaryKey()));
			}
			LanguageBundleAmendDialog dialog = new LanguageBundleAmendDialog(null, true);
			dialog.show(languages);

		} catch (Exception e) {
			Main.errorHandler(parent, Main.getString("fina2.title"), Main.getString("fina2.delete.language"));
		}
	}

	class DefaultLanguageAction extends AbstractAction {

		@Override
		public void actionPerformed(ActionEvent e) {
		}

	}

}
