/*
 * ScheduleAmendDialog.java
 *
 * Created on November 6, 2001, 1:06 PM
 */
package fina2.returns;

import java.awt.GridBagConstraints;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.transaction.UserTransaction;

import fina2.Main;
import fina2.bank.Bank;
import fina2.bank.BankHome;
import fina2.bank.BankPK;
import fina2.bank.SelectBankDialog;
import fina2.period.Period;
import fina2.period.PeriodHome;
import fina2.period.PeriodPK;
import fina2.period.PeriodType;
import fina2.period.PeriodTypeHome;
import fina2.period.SelectPeriodDialog;
import fina2.ui.UIManager;
import fina2.ui.table.TableRow;
import fina2.ui.table.TableRowImpl;

public class ScheduleAmendDialog extends javax.swing.JDialog {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;
	private fina2.Main main = fina2.Main.main;

	private TableRow tableRow;
	private Schedule schedule;
	private Bank bank;
	private ReturnDefinition definition;
	private Period period;

	private SelectReturnDefinitionDialog selectDefinitionDialog;
	private SelectBankDialog selectBankDialog;
	private SelectPeriodDialog selectPeriodDialog;

	/** Creates new form ScheduleAmendDialog */
	public ScheduleAmendDialog(java.awt.Frame parent, boolean modal) {
		super(parent, modal);

		selectDefinitionDialog = new SelectReturnDefinitionDialog(parent, true);
		selectBankDialog = new SelectBankDialog(parent, true);
		selectPeriodDialog = new SelectPeriodDialog(parent, true);

		ui.loadIcon("fina2.cancel", "cancel.gif");
		ui.loadIcon("fina2.help", "help.gif");
		ui.loadIcon("fina2.ok", "ok.gif");
		ui.loadIcon("fina2.lookup", "find.gif");

		initComponents();
	}

	public TableRow getTableRow() {
		return tableRow;
	}

	public void show(TableRow tableRow, boolean canAmend) {
		this.tableRow = tableRow;
		schedule = null;

		bankLookupButton.setEnabled(canAmend);
		definitionLookupButton.setEnabled(canAmend);
		periodLookupButton.setEnabled(canAmend);
		doaText.setEditable(canAmend);

		if (tableRow != null) {

			SchedulePK pk = (SchedulePK) tableRow.getPrimaryKey();

			try {
				InitialContext jndi = fina2.Main.getJndiContext();

				Object ref = jndi.lookup("fina2/returns/Schedule");
				ScheduleHome home = (ScheduleHome) PortableRemoteObject.narrow(
						ref, ScheduleHome.class);

				schedule = home.findByPrimaryKey(pk);

				ref = jndi.lookup("fina2/bank/Bank");
				BankHome bankHome = (BankHome) PortableRemoteObject.narrow(ref,
						BankHome.class);

				bank = bankHome.findByPrimaryKey(schedule.getBankPK());

				ref = jndi.lookup("fina2/returns/ReturnDefinition");
				ReturnDefinitionHome definitionHome = (ReturnDefinitionHome) PortableRemoteObject
						.narrow(ref, ReturnDefinitionHome.class);

				definition = definitionHome.findByPrimaryKey(schedule
						.getReturnDefinitionPK());

				ref = jndi.lookup("fina2/period/Period");
				PeriodHome periodHome = (PeriodHome) PortableRemoteObject
						.narrow(ref, PeriodHome.class);

				period = periodHome.findByPrimaryKey(schedule.getPeriodPK());

				ref = jndi.lookup("fina2/period/PeriodType");
				PeriodTypeHome periodTypeHome = (PeriodTypeHome) PortableRemoteObject
						.narrow(ref, PeriodTypeHome.class);

				PeriodType periodType = periodTypeHome.findByPrimaryKey(period
						.getType());

				bankText.setText(bank.getName(main.getLanguageHandle()));
				definitionText.setText(definition.getDescription(main
						.getLanguageHandle()));
				periodTypeText.setText(periodType.getDescription(main
						.getLanguageHandle()));
				periodNumberText.setText(period.getPeriodNumber());
				dateFromText.setText(period.getFromDate(main
						.getLanguageHandle()));
				dateToText.setText(period.getToDate(main.getLanguageHandle()));

				Integer dd = new Integer(schedule.getDelay());
				doaText.setText(dd.toString());

			} catch (Exception e) {
				Main.generalErrorHandler(e);
				return;
			}
		}
		setLocationRelativeTo(getParent());
		show();
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {// GEN-BEGIN:initComponents

		jPanel1 = new javax.swing.JPanel();
		jPanel2 = new javax.swing.JPanel();
		helpButton = new javax.swing.JButton();
		jPanel3 = new javax.swing.JPanel();
		okButton = new javax.swing.JButton();
		cancelButton = new javax.swing.JButton();
		jPanel4 = new javax.swing.JPanel();
		jLabel1 = new javax.swing.JLabel();
		bankText = new javax.swing.JTextField();
		jLabel2 = new javax.swing.JLabel();
		definitionText = new javax.swing.JTextField();
		jLabel3 = new javax.swing.JLabel();
		jPanel5 = new javax.swing.JPanel();
		periodTypeText = new javax.swing.JTextField();
		dateFromText = new javax.swing.JTextField();
		dateToText = new javax.swing.JTextField();
		periodNumberText = new javax.swing.JTextField();
		bankLookupButton = new javax.swing.JButton();
		definitionLookupButton = new javax.swing.JButton();
		periodLookupButton = new javax.swing.JButton();
		jLabel4 = new javax.swing.JLabel();
		jPanel6 = new javax.swing.JPanel();
		doaText = new javax.swing.JTextField();
		jLabel5 = new javax.swing.JLabel();

		setTitle(ui.getString("fina2.returns.schedule"));
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				closeDialog(evt);
			}
		});

		jPanel1.setLayout(new java.awt.BorderLayout());

		helpButton.setIcon(ui.getIcon("fina2.help"));
		helpButton.setFont(ui.getFont());
		helpButton.setText(ui.getString("fina2.help"));
		helpButton.setEnabled(false);
		jPanel2.add(helpButton);

		jPanel1.add(jPanel2, java.awt.BorderLayout.WEST);

		okButton.setIcon(ui.getIcon("fina2.ok"));
		okButton.setFont(ui.getFont());
		okButton.setText(ui.getString("fina2.ok"));
		okButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				okButtonActionPerformed(evt);
			}
		});

		jPanel3.add(okButton);

		cancelButton.setIcon(ui.getIcon("fina2.cancel"));
		cancelButton.setFont(ui.getFont());
		cancelButton.setText(ui.getString("fina2.cancel"));
		cancelButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				cancelButtonActionPerformed(evt);
			}
		});

		jPanel3.add(cancelButton);

		jPanel1.add(jPanel3, java.awt.BorderLayout.EAST);

		getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

		jPanel4.setLayout(new java.awt.GridBagLayout());

		jPanel4.setBorder(new javax.swing.border.EmptyBorder(
				new java.awt.Insets(30, 70, 30, 70)));
		jLabel1.setText(UIManager.formatedHtmlString(ui
				.getString("fina2.bank.bank")));
		jLabel1.setFont(ui.getFont());
		jPanel4.add(jLabel1, UIManager.getGridBagConstraints(0, 0, 10, 10, -1,
				-1, GridBagConstraints.EAST, -1, null));

		bankText.setEditable(false);
		bankText.setColumns(15);
		bankText.setFont(ui.getFont());
		jPanel4.add(bankText, UIManager.getGridBagConstraints(1, 0, -1, -1, -1,
				-1, -1, GridBagConstraints.HORIZONTAL, null));

		jLabel2.setText(UIManager.formatedHtmlString(ui
				.getString("fina2.returns.returnDefinition")));
		jLabel2.setFont(ui.getFont());
		jPanel4.add(jLabel2, UIManager.getGridBagConstraints(0, 1, 10, 10, -1,
				-1, GridBagConstraints.EAST, -1, null));

		definitionText.setEditable(false);
		definitionText.setColumns(15);
		definitionText.setFont(ui.getFont());
		jPanel4.add(definitionText, UIManager.getGridBagConstraints(1, 1, -1,
				-1, -1, -1, -1, GridBagConstraints.HORIZONTAL, null));

		jLabel3.setText(UIManager.formatedHtmlString(ui
				.getString("fina2.period.period")));
		jLabel3.setFont(ui.getFont());
		jPanel4.add(jLabel3, UIManager.getGridBagConstraints(0, 2, 10, 10, -1,
				-1, GridBagConstraints.EAST, -1, null));

		jPanel5.setLayout(new java.awt.GridBagLayout());

		periodTypeText.setEditable(false);
		periodTypeText.setColumns(10);
		periodTypeText.setFont(ui.getFont());
		jPanel5.add(periodTypeText, UIManager.getGridBagConstraints(0, 0, -1,
				-1, -1, -1, -1, -1, null));

		dateFromText.setEditable(false);
		dateFromText.setColumns(8);
		dateFromText.setFont(ui.getFont());
		jPanel5.add(dateFromText, UIManager.getGridBagConstraints(2, 0, -1, -1,
				-1, -1, -1, -1, new java.awt.Insets(0, 5, 0, 0)));

		dateToText.setEditable(false);
		dateToText.setColumns(8);
		dateToText.setFont(ui.getFont());
		jPanel5.add(dateToText, UIManager.getGridBagConstraints(3, 0, -1, -1,
				-1, -1, -1, -1, new java.awt.Insets(0, 5, 0, 0)));

		periodNumberText.setEditable(false);
		periodNumberText.setColumns(3);
		periodNumberText.setFont(ui.getFont());
		jPanel5.add(periodNumberText, UIManager.getGridBagConstraints(1, 0, -1,
				-1, -1, -1, -1, -1, new java.awt.Insets(0, 5, 0, 0)));
		jPanel4.add(jPanel5, UIManager.getGridBagConstraints(1, 2, -1, -1, -1,
				-1, -1, GridBagConstraints.BOTH, null));

		bankLookupButton.setIcon(ui.getIcon("fina2.lookup"));
		bankLookupButton.setFont(ui.getFont());
		bankLookupButton.setMargin(new java.awt.Insets(1, 1, 1, 1));
		bankLookupButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				bankLookupButtonActionPerformed(evt);
			}
		});
		jPanel4.add(bankLookupButton, UIManager.getGridBagConstraints(2, 0, -1,
				-1, -1, -1, -1, -1, new java.awt.Insets(0, 5, 0, 0)));

		definitionLookupButton.setIcon(ui.getIcon("fina2.lookup"));
		definitionLookupButton.setFont(ui.getFont());
		definitionLookupButton.setMargin(new java.awt.Insets(1, 1, 1, 1));
		definitionLookupButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						definitionLookupButtonActionPerformed(evt);
					}
				});
		jPanel4.add(definitionLookupButton, UIManager.getGridBagConstraints(2,
				1, -1, -1, -1, -1, -1, -1, new java.awt.Insets(0, 5, 0, 0)));

		periodLookupButton.setIcon(ui.getIcon("fina2.lookup"));
		periodLookupButton.setFont(ui.getFont());
		periodLookupButton.setMargin(new java.awt.Insets(1, 1, 1, 1));
		periodLookupButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						periodLookupButtonActionPerformed(evt);
					}
				});
		jPanel4.add(periodLookupButton, UIManager.getGridBagConstraints(2, 2,
				-1, -1, -1, -1, -1, -1, new java.awt.Insets(0, 5, 0, 0)));

		jLabel4.setText(UIManager.formatedHtmlString(ui
				.getString("fina2.returns.acceptableDelay")));
		jLabel4.setFont(ui.getFont());
		jPanel4.add(jLabel4, UIManager.getGridBagConstraints(0, 3, 10, 10, -1,
				-1, GridBagConstraints.EAST, -1, null));

		jPanel6.setLayout(new java.awt.BorderLayout());

		doaText.setColumns(5);
		doaText.setFont(ui.getFont());
		jPanel6.add(doaText, java.awt.BorderLayout.WEST);

		jLabel5.setText(ui.getString("fina2.returns.days"));
		jLabel5.setFont(ui.getFont());
		jPanel6.add(jLabel5, java.awt.BorderLayout.CENTER);

		jPanel4.add(jPanel6, UIManager.getGridBagConstraints(1, 3, -1, -1, -1,
				-1, -1, GridBagConstraints.BOTH, null));

		getContentPane().add(jPanel4, java.awt.BorderLayout.CENTER);

		pack();
	}// GEN-END:initComponents

	private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN
		// -
		// FIRST
		// :
		// event_cancelButtonActionPerformed
		dispose();
	}// GEN-LAST:event_cancelButtonActionPerformed

	private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN-
		// FIRST
		// :
		// event_okButtonActionPerformed

		try {
			Integer.valueOf(doaText.getText());
		} catch (Exception e) {
			Main.errorHandler(null, Main.getString("fina2.title"),
					Main.getString("fina2.returns.enterAcceptableDelay"));
			return;
		}

		if (bank == null) {
			ui.showMessageBox((java.awt.Frame) getParent(),
					ui.getString("fina2.title"),
					ui.getString("fina2.returns.bankRequired"));
			return;
		} else if (definition == null) {
			ui.showMessageBox((java.awt.Frame) getParent(),
					ui.getString("fina2.title"),
					ui.getString("fina2.returns.returnDefinitionRequired"));
			return;
		} else if (period == null) {
			ui.showMessageBox((java.awt.Frame) getParent(),
					ui.getString("fina2.title"),
					ui.getString("fina2.returns.periodRequired"));
			return;
		}

		try {
			InitialContext jndi = fina2.Main.getJndiContext();
			Object ref = jndi.lookup("fina2/returns/Schedule");
			ScheduleHome home = (ScheduleHome) PortableRemoteObject.narrow(ref,
					ScheduleHome.class);

			try {
				if (tableRow == null) {
					schedule = home.create((BankPK) bank.getPrimaryKey(),
							(ReturnDefinitionPK) definition.getPrimaryKey(),
							(PeriodPK) period.getPrimaryKey());
				}
			} catch (Exception e) {
				if (tableRow == null)
					schedule = null;
				Main.generalErrorHandler(e);
				return;
			}

			UserTransaction trans = main.getUserTransaction(jndi);
			try {
				trans.begin();

				schedule.setBankPK((BankPK) bank.getPrimaryKey());
				schedule.setReturnDefinitionPK((ReturnDefinitionPK) definition
						.getPrimaryKey());
				schedule.setPeriodPK((PeriodPK) period.getPrimaryKey());

				Integer del = new Integer(doaText.getText());
				schedule.setDelay(del.intValue());

				trans.commit();
			} catch (Exception e) {
				Main.generalErrorHandler(e);
				trans.rollback();
				if (tableRow == null) {
					schedule.remove();
					schedule = null;
				}
				return;
			}

			if (tableRow == null) {
				ref = jndi.lookup("fina2/returns/ReturnType");
				ReturnTypeHome typeHome = (ReturnTypeHome) PortableRemoteObject
						.narrow(ref, ReturnTypeHome.class);
				ReturnType returnType = typeHome.findByPrimaryKey(definition
						.getType());

				tableRow = new TableRowImpl(schedule.getPrimaryKey(), 7);
				tableRow.setValue(0, definition.getCode());
				tableRow.setValue(1,
						period.getFromDate(main.getLanguageHandle()));
				tableRow.setValue(2, period.getToDate(main.getLanguageHandle()));
				tableRow.setValue(3,
						definition.getDescription(main.getLanguageHandle()));
				tableRow.setValue(4, bank.getCode());
				tableRow.setValue(5, returnType.getCode());
				tableRow.setValue(6, doaText.getText());

			} else {
				if (bank != null) {
					tableRow.setValue(4, bank.getCode());
				}
				if (period != null) {
					tableRow.setValue(1,
							period.getFromDate(main.getLanguageHandle()));
					tableRow.setValue(2,
							period.getToDate(main.getLanguageHandle()));
				}
				if (definition != null) {
					ref = jndi.lookup("fina2/returns/ReturnType");
					ReturnTypeHome typeHome = (ReturnTypeHome) PortableRemoteObject
							.narrow(ref, ReturnTypeHome.class);
					ReturnType returnType = typeHome
							.findByPrimaryKey(definition.getType());

					tableRow.setValue(0, definition.getCode());
					tableRow.setValue(3,
							definition.getDescription(main.getLanguageHandle()));
					tableRow.setValue(5, returnType.getCode());
				}
				tableRow.setValue(6, doaText.getText());
			}
		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
		dispose();
	}// GEN-LAST:event_okButtonActionPerformed

	private void periodLookupButtonActionPerformed(
			java.awt.event.ActionEvent evt) {// GEN-FIRST:
		// event_periodLookupButtonActionPerformed
		selectPeriodDialog.show();
		TableRow row = selectPeriodDialog.getTableRow();
		if (row == null)
			return;
		PeriodPK pk = (PeriodPK) row.getPrimaryKey();
		try {
			InitialContext jndi = fina2.Main.getJndiContext();

			Object ref = jndi.lookup("fina2/period/Period");
			PeriodHome periodHome = (PeriodHome) PortableRemoteObject.narrow(
					ref, PeriodHome.class);

			period = periodHome.findByPrimaryKey(pk);

			ref = jndi.lookup("fina2/period/PeriodType");
			PeriodTypeHome periodTypeHome = (PeriodTypeHome) PortableRemoteObject
					.narrow(ref, PeriodTypeHome.class);

			PeriodType periodType = periodTypeHome.findByPrimaryKey(period
					.getType());

			periodTypeText.setText(periodType.getDescription(main
					.getLanguageHandle()));
			periodNumberText.setText(period.getPeriodNumber());
			dateFromText.setText(period.getFromDate(main.getLanguageHandle()));
			dateToText.setText(period.getToDate(main.getLanguageHandle()));

		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
	}// GEN-LAST:event_periodLookupButtonActionPerformed

	private void definitionLookupButtonActionPerformed(
			java.awt.event.ActionEvent evt) {// GEN-FIRST:
		// event_definitionLookupButtonActionPerformed
		selectDefinitionDialog.show();
		TableRow row = selectDefinitionDialog.getTableRow();
		if (row == null)
			return;
		ReturnDefinitionPK pk = (ReturnDefinitionPK) row.getPrimaryKey();
		try {
			InitialContext jndi = fina2.Main.getJndiContext();

			Object ref = jndi.lookup("fina2/returns/ReturnDefinition");
			ReturnDefinitionHome definitionHome = (ReturnDefinitionHome) PortableRemoteObject
					.narrow(ref, ReturnDefinitionHome.class);

			definition = definitionHome.findByPrimaryKey(pk);

			definitionText.setText(definition.getDescription(main
					.getLanguageHandle()));

		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
	}// GEN-LAST:event_definitionLookupButtonActionPerformed

	private void bankLookupButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN
		// -
		// FIRST
		// :
		// event_bankLookupButtonActionPerformed
		selectBankDialog.show();
		TableRow row = selectBankDialog.getTableRow();
		if (row == null)
			return;
		BankPK pk = (BankPK) row.getPrimaryKey();
		try {
			InitialContext jndi = fina2.Main.getJndiContext();

			Object ref = jndi.lookup("fina2/bank/Bank");
			BankHome bankHome = (BankHome) PortableRemoteObject.narrow(ref,
					BankHome.class);

			bank = bankHome.findByPrimaryKey(pk);

			bankText.setText(bank.getName(main.getLanguageHandle()));

		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
	}// GEN-LAST:event_bankLookupButtonActionPerformed

	/** Closes the dialog */
	private void closeDialog(java.awt.event.WindowEvent evt) {// GEN-FIRST:
		// event_closeDialog
		setVisible(false);
		dispose();
	}// GEN-LAST:event_closeDialog

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton bankLookupButton;
	private javax.swing.JTextField bankText;
	private javax.swing.JButton cancelButton;
	private javax.swing.JTextField dateFromText;
	private javax.swing.JTextField dateToText;
	private javax.swing.JButton definitionLookupButton;
	private javax.swing.JTextField definitionText;
	private javax.swing.JTextField doaText;
	private javax.swing.JButton helpButton;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JLabel jLabel2;
	private javax.swing.JLabel jLabel3;
	private javax.swing.JLabel jLabel4;
	private javax.swing.JLabel jLabel5;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JPanel jPanel4;
	private javax.swing.JPanel jPanel5;
	private javax.swing.JPanel jPanel6;
	private javax.swing.JButton okButton;
	private javax.swing.JButton periodLookupButton;
	private javax.swing.JTextField periodNumberText;
	private javax.swing.JTextField periodTypeText;
	// End of variables declaration//GEN-END:variables

}
